
#include "../ncview.includes.h"
#include "../ncview.defines.h"
#include "../ncview.protos.h"

#define OVERLAY_FILENAME_WIDGET_WIDTH	500

extern Widget		topLevel;
extern XtAppContext	x_app_context;
extern Options		options;

static Widget
	opt_popup_widget,
		opt_popupcanvas_widget,
			opt_overlays_box_widget,
				opt_overlays_label_widget,
				opt_do_overlays_button_widget,
				opt_overlays_filename_widget,
			opt_action_box_widget,
				opt_ok_widget,
				opt_cancel_widget;

static int opt_popup_done = FALSE, opt_popup_result;

static XEvent	opt_event;
static int	overlay_button_state = FALSE;

void opt_popup_callback		( Widget widget, XtPointer client_data, XtPointer call_data);
void overlays_button_callback	( Widget widget, XtPointer client_data, XtPointer call_data);
void overlays_filename_callback	( Widget widget, XtPointer client_data, XtPointer call_data);
void overlays_filename_callback ( Widget widget, XtPointer client_data, XtPointer call_data);

/*****************************************************************************/

	int
set_options( void )
{
	int	llx, lly, urx, ury;


	/********** Pop up the window **********/
	x_get_window_position( &llx, &lly, &urx, &ury );
	XtVaSetValues( opt_popup_widget, XtNx, llx+10, XtNy, (ury+lly)/2, NULL );
	XtPopup      ( opt_popup_widget, XtGrabExclusive );

	if( options.display_type == PseudoColor )
		x_set_windows_colormap_to_current( opt_popup_widget );

	/**********************************************
	 * Main loop for the options popup widget
	 */
	while( ! opt_popup_done ) {
		/* An event will cause opt_popup_done to become TRUE */
		XtAppNextEvent( x_app_context, &opt_event );
		XtDispatchEvent( &opt_event );
		}
	opt_popup_done = FALSE;

	/**********************************************/

	XtPopdown( opt_popup_widget );
	return( opt_popup_result );
}

	void
set_options_init()
{
	opt_popup_widget = XtVaCreatePopupShell(
		"Set Options",
		transientShellWidgetClass,
		topLevel,
		NULL );

	opt_popupcanvas_widget = XtVaCreateManagedWidget(
		"opt_popupcanvas",
		formWidgetClass,
		opt_popup_widget,
		NULL);

	opt_overlays_box_widget = XtVaCreateManagedWidget(
		"opt_overlays_box_widget",	
		boxWidgetClass,	
		opt_popupcanvas_widget,
		XtNorientation, XtorientHorizontal,
		NULL );

		opt_overlays_label_widget = XtVaCreateManagedWidget(
			"opt_overlays_label_widget",	
			labelWidgetClass,	
			opt_overlays_box_widget,
			XtNlabel, "Overlays:",
			XtNborderWidth, 0,
			NULL );
	
		opt_do_overlays_button_widget = XtVaCreateManagedWidget(
			"opt_do_overlays_button_widget",	
			toggleWidgetClass,	
			opt_overlays_box_widget,
			XtNlabel, "Off",
			NULL );
	
       		 XtAddCallback( opt_do_overlays_button_widget, XtNcallback,
			overlays_button_callback, (XtPointer)NULL );
	
		opt_overlays_filename_widget = XtVaCreateManagedWidget(
			"opt_overlays_filename_widget",	
			commandWidgetClass,	
			opt_overlays_box_widget,
			XtNwidth, OVERLAY_FILENAME_WIDGET_WIDTH,
			XtNjustify, XtJustifyLeft,
			NULL );
	
       		 XtAddCallback( opt_overlays_filename_widget, XtNcallback,
			overlays_filename_callback, (XtPointer)NULL );
	
	opt_action_box_widget = XtVaCreateManagedWidget(
		"opt_action_box_widget",	
		boxWidgetClass,	
		opt_popupcanvas_widget,
		XtNorientation, XtorientHorizontal,
		XtNfromVert, opt_overlays_box_widget,
		NULL );
	
		opt_ok_widget = XtVaCreateManagedWidget(
			"OK",	
			commandWidgetClass,	
			opt_action_box_widget,
			NULL );
	
       	 	XtAddCallback( opt_ok_widget, XtNcallback, 
			opt_popup_callback, (XtPointer)MESSAGE_OK );
	
		opt_cancel_widget = XtVaCreateManagedWidget(
			"Cancel",	
			commandWidgetClass,	
			opt_action_box_widget,
			NULL );

       	 	XtAddCallback( opt_cancel_widget, XtNcallback, 
			opt_popup_callback, (XtPointer)MESSAGE_CANCEL );

}

        void
opt_popup_callback( Widget widget, XtPointer client_data, XtPointer call_data)
{
	opt_popup_result = (int)client_data;
	opt_popup_done   = TRUE;
}

        void
overlays_filename_callback( Widget widget, XtPointer client_data, XtPointer call_data)
{
	overlay_select_filename();
}

        void
overlays_button_callback( Widget widget, XtPointer client_data, XtPointer call_data)
{
	if( overlay_button_state ) 
		overlay_button_state = FALSE;
	else
		overlay_button_state = TRUE;

	if( overlay_button_state ) {
        	XtVaSetValues( opt_do_overlays_button_widget, XtNlabel, "On", NULL );
		do_overlay();
		}
	else
		{
        	XtVaSetValues( opt_do_overlays_button_widget, XtNlabel, "Off", NULL );
		options.overlay->doit = FALSE;
		invalidate_all_saveframes();
		change_view( 0, FRAMES );
		}
}

	void
options_set_overlay_filename( char *fn )
{
        XtVaSetValues( opt_overlays_filename_widget, XtNlabel, fn, 
			XtNwidth, OVERLAY_FILENAME_WIDGET_WIDTH, NULL );
}

	void
options_set_overlay_state( int state )
{
	overlay_button_state = state;

	if( state )
        	XtVaSetValues( opt_do_overlays_button_widget, XtNstate, TRUE, 
        			XtNlabel, "On", NULL );
	else
        	XtVaSetValues( opt_do_overlays_button_widget, XtNstate, FALSE, 
        			XtNlabel, "Off", NULL );
}
