/*
 * Ncview by David W. Pierce.  A visual netCDF file viewer.
 * Copyright (C) 1993 through 2003 David W. Pierce
 *
 * This program  is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, Version 1, as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License, version 1, for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * David W. Pierce
 * 6259 Caminito Carrena
 * San Diego, CA  92122
 * pierce@cirrus.ucsd.edu
 */


#include "ncview.includes.h"
#include "ncview.defines.h"
#include "ncview.protos.h"

/* Program defaults in a easy-to-find place */
#define DEFAULT_INVERT_PHYSICAL	FALSE
#define DEFAULT_INVERT_COLORS	FALSE
#define DEFAULT_BLOWUP		1
#define DEFAULT_MIN_MAX_METHOD	MIN_MAX_METHOD_FAST
#define DEFAULT_N_COLORS	200
#define DEFAULT_PRIVATE_CMAP	FALSE
#define DEFAULT_BLOWUP_TYPE	BLOWUP_BILINEAR
#define DEFAULT_SAVEFRAMES	TRUE
#define DEFAULT_NO_AUTOFLIP	FALSE

Options	  options;
NCVar	  *variables;
ncv_pixel *pixel_transform;
FrameStore framestore;

int get_cmaps_from_dir( char *dir_name );

	int
main( int argc, char **argv )
{
	Stringlist *input_files;
Stringlist *files, *dirs;

	initialize_misc             ();
	in_parse_args               ( &argc, argv );
	input_files = parse_options ( argc,  argv );
	determine_file_type         ( input_files );

	options.window_title = input_files->string;
	options.blowup       = 1;

	/* this routine sets up the 'variables' structure */
	initialize_file_interface   ( input_files );

	if( n_vars_in_list( variables ) == 0 ) {
		fprintf( stderr, "no displayable variables found!\n" );
		exit( -1 );
		}

	initialize_display_interface(); 

	/* If there is only one variable, make it the active one */
	if( n_vars_in_list( variables ) == 1 ) {
		/* set_scan_variable     ( variables       ); */
		in_indicate_active_var( variables->name );
		}

	print_init();
	overlay_init();

files=NULL;
dirs=NULL;
fs_list_dir( &files, &dirs );
	process_user_input();

	return(0);
}

	Stringlist *
parse_options( int argc, char *argv[] )
{
	int	i;
	Stringlist *file_list = NULL;

	for( i=1; i<argc; i++ ) {
		if( argv[i][0] == '-' ) {
			/* found an entry that is in option syntax */
			if( strncmp( argv[i], "-min", 4 ) == 0 ) {
				if( strncmp( argv[i+1], "fast", 4 ) == 0 ) {
					options.min_max_method  = MIN_MAX_METHOD_FAST;
					i++;
					}
				else if( strncmp( argv[i+1], "med", 3 ) == 0 ) {
					options.min_max_method  = MIN_MAX_METHOD_MED;
					i++;
					}
				else if( strncmp( argv[i+1], "slow", 4 ) == 0 ) {
					options.min_max_method  = MIN_MAX_METHOD_SLOW;
					i++;
					}
				else if( strncmp( argv[i+1], "exh", 3 ) == 0 ) {
					options.min_max_method  = MIN_MAX_METHOD_EXHAUST;
					i++;
					}
				else if( strncmp( argv[i+1], "all", 3 ) == 0 ) {
					options.min_max_method  = MIN_MAX_METHOD_EXHAUST;
					i++;
					}
				else
					{
					fprintf( stderr, "unrecognizied option: %s %s\n",
						argv[i], argv[i+1] );
					/* doesn't return */
					useage();
					}
				}
			else if( strncmp( argv[i], "-w", 2 ) == 0 ) {
				print_no_warranty();
				exit( 0 );
				}

			else if( strncmp( argv[i], "-pri", 4 ) == 0 )
				options.private_colormap = TRUE;

			else if( strncmp( argv[i], "-deb", 4 ) == 0 )
				options.debug = TRUE;

			else if( strncmp( argv[i], "-beep", 5 ) == 0 )
				options.beep_on_restart = TRUE;

			else if( strncmp( argv[i], "-fra", 4 ) == 0 )
				options.dump_frames = TRUE;

			else if( strncmp( argv[i], "-small", 6 ) == 0 )
				options.small = TRUE;

			else if( strncmp( argv[i], "-ext", 4 ) == 0 )
				options.want_extra_info = TRUE;

			else if( strncmp( argv[i], "-mti", 3 ) == 0 ) {
				options.window_title = argv[i+1];
				i++;
				}

			else if( strncmp( argv[i], "-noauto", 7 ) == 0 )
				options.no_autoflip = TRUE;

			else if( strncmp( argv[i], "-no1d", 5 ) == 0 )
				options.no_1d_vars = TRUE;

			else if( strncmp( argv[i], "-show_sel", 9 ) == 0 )
				options.show_sel = TRUE;

			else if( strncmp( argv[i], "-no_char_dim", 12 ) == 0 )
				options.no_char_dims = TRUE;

			else if( strncmp( argv[i], "-notconv", 8 ) == 0 )
				options.t_conv = FALSE;

			else if( strncmp( argv[i], "-c", 2 ) == 0 ) {
				print_copying();
				exit( 0 );
				}

			else if( strncmp( argv[i], "-nc", 3 ) == 0 ) {
				sscanf( argv[i+1], "%d", &(options.n_colors) );
				if( options.n_colors > 255 ) {
					fprintf( stderr, "maximum number of colors is currently 255\n" );
					exit( -1 );
					}
				i++;
				}

			else	/* put other options here */
				{
				fprintf( stderr, "unrecognizied option: %s\n", argv[i] );
				/* doesn't return */
				useage();
				}
			}
		else /* found an entry which is NOT in option syntax -- assume a filename */
			add_to_stringlist( &file_list, argv[i], NULL );
		} /* end of i loop through argv's */

	return( file_list );
}

	void
initialize_misc()
{
	print_disclaimer();

	udu_utinit( NULL );
	options.invert_physical  = DEFAULT_INVERT_PHYSICAL;
	options.invert_colors    = DEFAULT_INVERT_COLORS;
	options.blowup           = DEFAULT_BLOWUP;
	options.min_max_method   = DEFAULT_MIN_MAX_METHOD;
	options.transform        = TRANSFORM_NONE;
	options.n_colors 	 = DEFAULT_N_COLORS;
	options.private_colormap = DEFAULT_PRIVATE_CMAP;
	options.debug		 = FALSE;
	options.show_sel	 = FALSE;
	options.want_extra_info  = FALSE;
	options.beep_on_restart  = FALSE;
	options.small  		 = FALSE;
	options.blowup_type      = DEFAULT_BLOWUP_TYPE;
	options.save_frames      = DEFAULT_SAVEFRAMES;
	options.no_autoflip      = DEFAULT_NO_AUTOFLIP;
	options.t_conv      	 = TRUE;
	options.varsel_style	 = VARSEL_LIST;
	options.dump_frames	 = FALSE;

	options.overlay          = (OverlayOptions *)malloc( sizeof( OverlayOptions ));
	options.overlay->doit    = FALSE;
	options.overlay->overlay = NULL;

	framestore.frame = NULL;
	framestore.valid = FALSE;
}

	void
initialize_colormaps()
{
	int	n_colormaps = 0;
	char	*ncview_base_dir;

	/* Colormaps are contained in files with the extension of ".ncmap", 
	 * and can live in the following places:
	 *  1) NCVIEW_LIB_DIR, which is determined at installation time.
	 *     	A reasonable choice is "/usr/local/lib/ncview".
	 *  2) In a directory named by the environmental variable
	 *	"NCVIEWBASE".
	 *  3) If there is no environmental variable "NCVIEWBASE", then
	 *     	in $HOME.
	 *  4) In the current working directory.
	 */

#ifdef NCVIEW_LIB_DIR
	n_colormaps  = get_cmaps_from_dir( NCVIEW_LIB_DIR );
#endif
	ncview_base_dir = (char *)getenv( "NCVIEWBASE" );
	if( ncview_base_dir == NULL )
		ncview_base_dir = (char *)getenv( "HOME" );

	if( ncview_base_dir != NULL )
		n_colormaps += get_cmaps_from_dir( ncview_base_dir );

	n_colormaps += get_cmaps_from_dir( "." );

	if( n_colormaps == 0 ) {
		fprintf( stderr, 
			"ncview: warning: no .ncmap (colormap) files found\n");
		create_default_colormap();
		}
}

	int
get_cmaps_from_dir( char *dir_name )
{
	DIR		*ncdir = NULL;
	struct dirent	*dir_entry;
	int		n_colormaps = 0;

	if( options.debug ) 
		fprintf( stderr, "Getting colormaps from dir >%s<\n", dir_name );

	ncdir     = opendir( dir_name );
	if( ncdir == NULL ) 
		return( 0 );
	dir_entry = readdir( ncdir    );
	while( dir_entry != NULL ) {
		/* Additional allowed filenames as per suggestion by 
		 * Arlindo da Silva for his Windows port
		 */
		if( (strstr( dir_entry->d_name, ".NCM"   ) != NULL) ||
		    (strstr( dir_entry->d_name, ".ncm"   ) != NULL) ||
		    (strstr( dir_entry->d_name, ".ncmap" ) != NULL) ) {
			init_cmap_from_file( dir_name, dir_entry->d_name );
			n_colormaps++;
			}
		dir_entry = readdir( ncdir );
		}
	closedir( ncdir );

	return( n_colormaps );
}

	void
init_cmap_from_file( char *dir_name, char *file_name )
{
	char 	*colormap_name;
	FILE	*cmap_file;
	int	i, nentries, r_entry, g_entry, b_entry;
	char	line[ 128 ], long_file_name[ 2048 ];
	unsigned char r[256], g[256], b[256];

	if( options.debug ) 
		fprintf( stderr, "    ... initting cmap >%s<\n", file_name );

	/* Colormap name is the file name without the '.ncmap' extension */
	colormap_name = (char *)malloc( strlen(file_name)-5 );
	strncpy( colormap_name, file_name, strlen(file_name)-6 );
	*(colormap_name + strlen(file_name)-6) = '\0';

	/* Read in the r, g, b values */
	sprintf( long_file_name, "%s/%s", dir_name, file_name );
	if( (cmap_file = fopen( long_file_name, "r" )) == NULL ) {
		fprintf( stderr, "ncview.c: init_cmap_from_file: error " );
		fprintf( stderr, "opening file %s\n", long_file_name );
		return;
		}
	for( i=0; i<256; i++ ) {
		if( fgets( line, 128, cmap_file ) == NULL ) {
			fprintf( stderr, "ncview: init_cmap_from_file: file %s finished ",
					long_file_name );
			fprintf( stderr, "on line %d, but should have 256 lines\n", i+1 );
			exit( -1 );
			}

		nentries = sscanf( line, "%d %d %d", &r_entry, &g_entry, &b_entry );
		if( nentries != 3 ) {
			fprintf( stderr, "ncview: init_cmap_from_file: incorrect number " );
			fprintf( stderr, "of entries on the line.  Should be 3\n" );
			fprintf( stderr, "file %s, line %d\n", long_file_name, i+1 );
			return;
			}

		if( check( r_entry, 0, 255 ) < 0 ) {
			fprintf( stderr, "ncview: init_cmap_from_file: first entry (red) " );
			fprintf( stderr, "is outside valid limits of 0 to 255.\n" );
			fprintf( stderr, "file %s, line %d\n", long_file_name, i+1 );
			return;
			}

		if( check( g_entry, 0, 255 ) < 0 ) {
			fprintf( stderr, "ncview: init_cmap_from_file: second entry (green) " );
			fprintf( stderr, "is outside valid limits of 0 to 255.\n" );
			fprintf( stderr, "file %s, line %d\n", long_file_name, i+1 );
			return;
			}

		if( check( b_entry, 0, 255 ) < 0 ) {
			fprintf( stderr, "ncview: init_cmap_from_file: third entry (blue) " );
			fprintf( stderr, "is outside valid limits of 0 to 255.\n" );
			fprintf( stderr, "file %s, line %d\n", long_file_name, i+1 );
			return;
			}
		r[i] = (unsigned char)r_entry;
		g[i] = (unsigned char)g_entry;
		b[i] = (unsigned char)b_entry;
		}

	in_create_colormap( colormap_name, r, g, b );
}

	void
initialize_file_interface( Stringlist *input_files )
{
	int	idim;
	NCVar	*var;

	while( input_files != NULL )
		{
		fi_initialize( input_files->string );
		input_files = input_files->next;
		}

	/* Get the effective dimensionality of all the vars.
	 * Can't do this before we have read in all of the
	 * input file.
	 */
	var = variables;
	while( var != NULL ) {
		var->effective_dimensionality = 0;
		for( idim=0; idim<var->n_dims; idim++ ) 
			if( *(var->size + idim) > 1 )
				var->effective_dimensionality++;
		if( options.debug ) 
			fprintf( stderr, "variable %s had effective_dimensionality of %d\n",
				var->name, var->effective_dimensionality );
		var = var->next;
		}
}

	void
initialize_display_interface()
{
	initialize_colormaps();
	in_initialize       ();
}

	void
process_user_input()
{
	/* This call never returns, as it loops, handling user interface events */
	in_process_user_input();
}

/* Only correct exit point for 'ncview' */
	void
quit_app()
{
	exit( 0 );
}

	int
check( int val, int min, int max )
{
	if( (val >= min) && (val <= max) )
		return( 0 );
	else
		return( -1 );
}

/* Make a quickie colormap in case none is found */
	void
create_default_colormap()
{
	ncv_pixel	r[256], g[256], b[256];
	int		i;

	for( i=0; i<256; i++ ) {
		r[i] = i;
		g[i] = 255 - abs(i-128);
		b[i] = 255-i;
		}

	in_create_colormap( "default", r, g, b );
}

	void
useage()
{
fprintf( stderr, "\nuseage:\n" );
fprintf( stderr, "ncview [options] datafiles\n" );
fprintf( stderr, "\n" );
fprintf( stderr, "Options\n" );
fprintf( stderr, "	-minmax: selects how rapidly minimum and maximum\n" );
fprintf( stderr, "		values in the data files will be determined;\n" );
fprintf( stderr, "		by scanning every third time entry (\"-minmax fast\"),\n" );
fprintf( stderr, "		every fifth time entry (\"-minmax med\"), every tenth\n" );
fprintf( stderr, "		(\"-minmax slow\"), or all entries (\"-minmax all\").\n" );
fprintf( stderr, "	-frames: Dump out PPM images (to make a movie, for instance)\n" );
fprintf( stderr, "	-nc: 	Specify number of colors to use.\n" );
fprintf( stderr, "	-no1d: 	Do NOT allow 1-D variables to be displayed.\n" );
fprintf( stderr, "	-private: Use a private colormap.\n" );
fprintf( stderr, "	-debug: Print lots of debugging info.\n" );
fprintf( stderr, "	-beep: 	Ring the bell when the movie restarts at frame zero.\n" );
fprintf( stderr, "	-extra: Put some extra information on the display window.\n" );
fprintf( stderr, "	-mtitle: My title to use on the display window.\n" );
fprintf( stderr, "	-noautoflip: Do not automatically flip image, even\n" );
fprintf( stderr, "		if dimensions indicate that it would make sense.\n" );
fprintf( stderr, "	-w: 	print the lack-of-warranty blurb.\n" );
fprintf( stderr, "	-small: Keep popup window as small as possible by default.\n" );
fprintf( stderr, "	-c: 	print the copying policy.\n" );
fprintf( stderr, "datafiles:\n" );
fprintf( stderr, "	You can have up to 32 of these.  They must all be in\n" );
fprintf( stderr, "	the same general format, or have different variables in\n" );
fprintf( stderr, "	them.  Ncview tries its best under such circumstances.\n" );

exit( -1 );
}

	void
print_disclaimer()
{
fprintf( stderr, "%s\n", PROGRAM_ID );
fprintf( stderr, "http://meteora.ucsd.edu:80/~pierce/ncview_home_page.html\n" );
fprintf( stderr, "Copyright (C) 1993 through 2003, David W. Pierce\n" );
fprintf( stderr, "Ncview comes with ABSOLUTELY NO WARRANTY; for details type `ncview -w'.\n" );
fprintf( stderr, "This is free software; type `ncview -c' for redistribution details.\n\n" );
}

	void
print_no_warranty()
{
printf( "\n The program `ncview' is Copyright (C) 1993 through 2003 David W. Pierce, and\n" );
printf( "is subject to the terms and conditions of the Gnu General Public License,\n" );
printf( "Version 1. For information on copying, modifying, or distributing `ncview',\n" );
printf( "type `ncview -c'.\n" );
printf( "\n" );
printf( "  This License Agreement applies to any program or other work which \n" );
printf( "contains a notice placed by the copyright holder saying it may be \n" );
printf( "distributed under the terms of this General Public License.  The\n" );
printf( "\"Program\", below, refers to any such program or work, and a \"work based\n" );
printf( "on the Program\" means either the Program or any work containing the\n" );
printf( "Program or a portion of it, either verbatim or with modifications.  Each\n" );
printf( "licensee is addressed as \"you\".\n" );
printf( "\n" );
printf( "                            NO WARRANTY\n" );
printf( "\n" );
printf( "  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY\n" );
printf( "FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN\n" );
printf( "OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES\n" );
printf( "PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED\n" );
printf( "OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF\n" );
printf( "MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS\n" );
printf( "TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE\n" );
printf( "PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,\n" );
printf( "REPAIR OR CORRECTION.\n" );
printf( "\n" );
printf( "  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING\n" );
printf( "WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR\n" );
printf( "REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,\n" );
printf( "INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING\n" );
printf( "OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n" );
printf( "TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY\n" );
printf( "YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER\n" );
printf( "PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE\n" );
printf( "POSSIBILITY OF SUCH DAMAGES.\n" );
}

	void 
print_copying()
{
printf( "  The program `ncview' is Copyright (C) 1993 through 2003, David W. Pierce, and \n" );
printf( "is subject to the terms and conditions of the Gnu General Public License,\n" );
printf( "Version 1.  Ncview comes with NO WARRANTY; for further information, type\n" );
printf( "`ncview -w'.\n" );
printf( "\n" );
printf( "		    GNU GENERAL PUBLIC LICENSE\n");
printf( "		     Version 1, February 1989\n");
printf( "\n");
printf( " Copyright (C) 1989 Free Software Foundation, Inc.\n");
printf( "                    675 Mass Ave, Cambridge, MA 02139, USA\n");
printf( " Everyone is permitted to copy and distribute verbatim copies\n");
printf( " of this license document, but changing it is not allowed.\n");
printf( "\n");
printf( "			    Preamble\n");
printf( "\n");
printf( "  The license agreements of most software companies try to keep users\n");
printf( "at the mercy of those companies.  By contrast, our General Public\n");
printf( "License is intended to guarantee your freedom to share and change free\n");
printf( "software--to make sure the software is free for all its users.  The\n");
printf( "General Public License applies to the Free Software Foundation's\n");
printf( "software and to any other program whose authors commit to using it.\n");
printf( "You can use it for your programs, too.\n");
printf( "\n");
printf( "  When we speak of free software, we are referring to freedom, not\n");
printf( "price.  Specifically, the General Public License is designed to make\n");
printf( "sure that you have the freedom to give away or sell copies of free\n");
printf( "software, that you receive source code or can get it if you want it,\n");
printf( "that you can change the software or use pieces of it in new free\n");
printf( "programs; and that you know you can do these things.\n");
printf( "\n");
printf( "  To protect your rights, we need to make restrictions that forbid\n");
printf( "anyone to deny you these rights or to ask you to surrender the rights.\n");
printf( "These restrictions translate to certain responsibilities for you if you\n");
printf( "distribute copies of the software, or if you modify it.\n");
printf( "\n");
printf( "  For example, if you distribute copies of a such a program, whether\n");
printf( "gratis or for a fee, you must give the recipients all the rights that\n");
printf( "you have.  You must make sure that they, too, receive or can get the\n");
printf( "source code.  And you must tell them their rights.\n");
printf( "\n");
printf( "  We protect your rights with two steps: (1) copyright the software, and\n");
printf( "(2) offer you this license which gives you legal permission to copy,\n");
printf( "distribute and/or modify the software.\n");
printf( "\n");
printf( "  Also, for each author's protection and ours, we want to make certain\n");
printf( "that everyone understands that there is no warranty for this free\n");
printf( "software.  If the software is modified by someone else and passed on, we\n");
printf( "want its recipients to know that what they have is not the original, so\n");
printf( "that any problems introduced by others will not reflect on the original\n");
printf( "authors' reputations.\n");
printf( "\n");
printf( "  The precise terms and conditions for copying, distribution and\n");
printf( "modification follow.\n");
printf( "\n");
printf( "		    GNU GENERAL PUBLIC LICENSE\n");
printf( "   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION\n");
printf( "\n");
printf( "  0. This License Agreement applies to any program or other work which\n");
printf( "contains a notice placed by the copyright holder saying it may be\n");
printf( "distributed under the terms of this General Public License.  The\n");
printf( "\"Program\", below, refers to any such program or work, and a \"work based\n");
printf( "on the Program\" means either the Program or any work containing the\n");
printf( "Program or a portion of it, either verbatim or with modifications.  Each\n");
printf( "licensee is addressed as \"you\".\n");
printf( "\n");
printf( "  1. You may copy and distribute verbatim copies of the Program's source\n");
printf( "code as you receive it, in any medium, provided that you conspicuously and\n");
printf( "appropriately publish on each copy an appropriate copyright notice and\n");
printf( "disclaimer of warranty; keep intact all the notices that refer to this\n");
printf( "General Public License and to the absence of any warranty; and give any\n");
printf( "other recipients of the Program a copy of this General Public License\n");
printf( "along with the Program.  You may charge a fee for the physical act of\n");
printf( "transferring a copy.\n");
printf( "\n");
printf( "  2. You may modify your copy or copies of the Program or any portion of\n");
printf( "it, and copy and distribute such modifications under the terms of Paragraph\n");
printf( "1 above, provided that you also do the following:\n");
printf( "\n");
printf( "    a) cause the modified files to carry prominent notices stating that\n");
printf( "    you changed the files and the date of any change; and\n");
printf( "\n");
printf( "    b) cause the whole of any work that you distribute or publish, that\n");
printf( "    in whole or in part contains the Program or any part thereof, either\n");
printf( "    with or without modifications, to be licensed at no charge to all\n");
printf( "    third parties under the terms of this General Public License (except\n");
printf( "    that you may choose to grant warranty protection to some or all\n");
printf( "    third parties, at your option).\n");
printf( "\n");
printf( "    c) If the modified program normally reads commands interactively when\n");
printf( "    run, you must cause it, when started running for such interactive use\n");
printf( "    in the simplest and most usual way, to print or display an\n");
printf( "    announcement including an appropriate copyright notice and a notice\n");
printf( "    that there is no warranty (or else, saying that you provide a\n");
printf( "    warranty) and that users may redistribute the program under these\n");
printf( "    conditions, and telling the user how to view a copy of this General\n");
printf( "    Public License.\n");
printf( "\n");
printf( "    d) You may charge a fee for the physical act of transferring a\n");
printf( "    copy, and you may at your option offer warranty protection in\n");
printf( "    exchange for a fee.\n");
printf( "\n");
printf( "Mere aggregation of another independent work with the Program (or its\n");
printf( "derivative) on a volume of a storage or distribution medium does not bring\n");
printf( "the other work under the scope of these terms.\n");
printf( "\n");
printf( "  3. You may copy and distribute the Program (or a portion or derivative of\n");
printf( "it, under Paragraph 2) in object code or executable form under the terms of\n");
printf( "Paragraphs 1 and 2 above provided that you also do one of the following:\n");
printf( "\n");
printf( "    a) accompany it with the complete corresponding machine-readable\n");
printf( "    source code, which must be distributed under the terms of\n");
printf( "    Paragraphs 1 and 2 above; or,\n");
printf( "\n");
printf( "    b) accompany it with a written offer, valid for at least three\n");
printf( "    years, to give any third party free (except for a nominal charge\n");
printf( "    for the cost of distribution) a complete machine-readable copy of the\n");
printf( "    corresponding source code, to be distributed under the terms of\n");
printf( "    Paragraphs 1 and 2 above; or,\n");
printf( "\n");
printf( "    c) accompany it with the information you received as to where the\n");
printf( "    corresponding source code may be obtained.  (This alternative is\n");
printf( "    allowed only for noncommercial distribution and only if you\n");
printf( "    received the program in object code or executable form alone.)\n");
printf( "\n");
printf( "Source code for a work means the preferred form of the work for making\n");
printf( "modifications to it.  For an executable file, complete source code means\n");
printf( "all the source code for all modules it contains; but, as a special\n");
printf( "exception, it need not include source code for modules which are standard\n");
printf( "libraries that accompany the operating system on which the executable\n");
printf( "file runs, or for standard header files or definitions files that\n");
printf( "accompany that operating system.\n");
printf( "\n");
printf( "  4. You may not copy, modify, sublicense, distribute or transfer the\n");
printf( "Program except as expressly provided under this General Public License.\n");
printf( "Any attempt otherwise to copy, modify, sublicense, distribute or transfer\n");
printf( "the Program is void, and will automatically terminate your rights to use\n");
printf( "the Program under this License.  However, parties who have received\n");
printf( "copies, or rights to use copies, from you under this General Public\n");
printf( "License will not have their licenses terminated so long as such parties\n");
printf( "remain in full compliance.\n");
printf( "\n");
printf( "  5. By copying, distributing or modifying the Program (or any work based\n");
printf( "on the Program) you indicate your acceptance of this license to do so,\n");
printf( "and all its terms and conditions.\n");
printf( "\n");
printf( "  6. Each time you redistribute the Program (or any work based on the\n");
printf( "Program), the recipient automatically receives a license from the original\n");
printf( "licensor to copy, distribute or modify the Program subject to these\n");
printf( "terms and conditions.  You may not impose any further restrictions on the\n");
printf( "recipients' exercise of the rights granted herein.\n");
printf( "\n");
printf( "  7. The Free Software Foundation may publish revised and/or new versions\n");
printf( "of the General Public License from time to time.  Such new versions will\n");
printf( "be similar in spirit to the present version, but may differ in detail to\n");
printf( "address new problems or concerns.\n");
printf( "\n");
printf( "Each version is given a distinguishing version number.  If the Program\n");
printf( "specifies a version number of the license which applies to it and \"any\n");
printf( "later version\", you have the option of following the terms and conditions\n");
printf( "either of that version or of any later version published by the Free\n");
printf( "Software Foundation.  If the Program does not specify a version number of\n");
printf( "the license, you may choose any version ever published by the Free Software\n");
printf( "Foundation.\n");
printf( "\n");
printf( "  8. If you wish to incorporate parts of the Program into other free\n");
printf( "programs whose distribution conditions are different, write to the author\n");
printf( "to ask for permission.  For software which is copyrighted by the Free\n");
printf( "Software Foundation, write to the Free Software Foundation; we sometimes\n");
printf( "make exceptions for this.  Our decision will be guided by the two goals\n");
printf( "of preserving the free status of all derivatives of our free software and\n");
printf( "of promoting the sharing and reuse of software generally.\n");
printf( "\n");
printf( "			    NO WARRANTY\n");
printf( "\n");
printf( "  9. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY\n");
printf( "FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN\n");
printf( "OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES\n");
printf( "PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED\n");
printf( "OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF\n");
printf( "MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS\n");
printf( "TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE\n");
printf( "PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,\n");
printf( "REPAIR OR CORRECTION.\n");
printf( "\n");
printf( "  10. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING\n");
printf( "WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR\n");
printf( "REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,\n");
printf( "INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING\n");
printf( "OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n");
printf( "TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY\n");
printf( "YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER\n");
printf( "PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE\n");
printf( "POSSIBILITY OF SUCH DAMAGES.\n");
printf( "\n");
printf( "		     END OF TERMS AND CONDITIONS\n");
}
